@@ -14,28 +14,32 @@ from __future__ import absolute_import
 
 import collections
 import inspect
-from io import open
+import io
 import os
 import os.path
 import re
 import sys
 import warnings
 
-from six import string_types, text_type
+from six import string_types, text_type, PY2
 
-from _sass import OUTPUT_STYLES, compile_filename, compile_string
+import _sass
+from sassutils._compat import collections_abc
 
-__all__ = ('MODES', 'OUTPUT_STYLES', 'SOURCE_COMMENTS', 'CompileError',
-           'SassColor', 'SassError', 'SassFunction', 'SassList', 'SassMap',
-           'SassNumber', 'SassWarning', 'and_join', 'compile')
-__version__ = '0.9.3'
+__all__ = (
+    'MODES', 'OUTPUT_STYLES', 'SOURCE_COMMENTS', 'CompileError', 'SassColor',
+    'SassError', 'SassFunction', 'SassList', 'SassMap', 'SassNumber',
+    'SassWarning', 'and_join', 'compile', 'libsass_version',
+)
+__version__ = '0.19.3'
+libsass_version = _sass.libsass_version
 
 
-#: (:class:`collections.Mapping`) The dictionary of output styles.
+#: (:class:`collections.abc.Mapping`) The dictionary of output styles.
 #: Keys are output name strings, and values are flag integers.
-OUTPUT_STYLES = OUTPUT_STYLES
+OUTPUT_STYLES = _sass.OUTPUT_STYLES
 
-#: (:class:`collections.Mapping`) The dictionary of source comments styles.
+#: (:class:`collections.abc.Mapping`) The dictionary of source comments styles.
 #: Keys are mode names, and values are corresponding flag integers.
 #:
 #: .. versionadded:: 0.4.0
@@ -43,14 +47,24 @@ OUTPUT_STYLES = OUTPUT_STYLES
 #: .. deprecated:: 0.6.0
 SOURCE_COMMENTS = {'none': 0, 'line_numbers': 1, 'default': 1, 'map': 2}
 
-#: (:class:`collections.Set`) The set of keywords :func:`compile()` can take.
-MODES = set(['string', 'filename', 'dirname'])
+#: (:class:`frozenset`) The set of keywords :func:`compile()` can take.
+MODES = frozenset(('string', 'filename', 'dirname'))
+
+
+def to_native_s(s):
+    if isinstance(s, bytes) and not PY2:  # pragma: no cover (py3)
+        s = s.decode('UTF-8')
+    elif isinstance(s, text_type) and PY2:  # pragma: no cover (py2)
+        s = s.encode('UTF-8')
+    return s
 
 
 class CompileError(ValueError):
     """The exception type that is raised by :func:`compile()`.
     It is a subtype of :exc:`exceptions.ValueError`.
     """
+    def __init__(self, msg):
+        super(CompileError, self).__init__(to_native_s(msg))
 
 
 def mkdirp(path):
@@ -69,9 +83,9 @@ class SassFunction(object):
     :param name: the function name
     :type name: :class:`str`
     :param arguments: the argument names
-    :type arguments: :class:`collections.Sequence`
+    :type arguments: :class:`collections.abc.Sequence`
     :param callable_: the actual function to be called
-    :type callable_: :class:`collections.Callable`
+    :type callable_: :class:`collections.abc.Callable`
 
     .. versionadded:: 0.7.0
 
@@ -93,14 +107,24 @@ class SassFunction(object):
         :rtype: :class:`SassFunction`
 
         """
-        argspec = inspect.getargspec(lambda_)
-        if argspec.varargs or argspec.keywords or argspec.defaults:
-            raise TypeError(
-                'functions cannot have starargs or defaults: {0} {1}'.format(
-                    name, lambda_
-                )
+        if PY2:  # pragma: no cover
+            a = inspect.getargspec(lambda_)
+            varargs, varkw, defaults, kwonlyargs = (
+                a.varargs, a.keywords, a.defaults, None,
             )
-        return cls(name, argspec.args, lambda_)
+        else:  # pragma: no cover
+            a = inspect.getfullargspec(lambda_)
+            varargs, varkw, defaults, kwonlyargs = (
+                a.varargs, a.varkw, a.defaults, a.kwonlyargs,
+            )
+
+        if varargs or varkw or defaults or kwonlyargs:
+            raise TypeError(
+                'functions cannot have starargs or defaults: {} {}'.format(
+                    name, lambda_,
+                ),
+            )
+        return cls(name, a.args, lambda_)
 
     @classmethod
     def from_named_function(cls, function):
@@ -120,9 +144,11 @@ class SassFunction(object):
     def __init__(self, name, arguments, callable_):
         if not isinstance(name, string_types):
             raise TypeError('name must be a string, not ' + repr(name))
-        elif not isinstance(arguments, collections.Sequence):
-            raise TypeError('arguments must be a sequence, not ' +
-                            repr(arguments))
+        elif not isinstance(arguments, collections_abc.Sequence):
+            raise TypeError(
+                'arguments must be a sequence, not ' +
+                repr(arguments),
+            )
         elif not callable(callable_):
             raise TypeError(repr(callable_) + ' is not callable')
         self.name = name
@@ -135,7 +161,7 @@ class SassFunction(object):
     @property
     def signature(self):
         """Signature string of the function."""
-        return '{0}({1})'.format(self.name, ', '.join(self.arguments))
+        return '{}({})'.format(self.name, ', '.join(self.arguments))
 
     def __call__(self, *args, **kwargs):
         return self.callable_(*args, **kwargs)
@@ -144,16 +170,83 @@ class SassFunction(object):
         return self.signature
 
 
+def _normalize_importer_return_value(result):
+    # An importer must return an iterable of iterables of 1-3 stringlike
+    # objects
+    if result is None:
+        return result
+
+    def _to_importer_result(single_result):
+        single_result = tuple(single_result)
+        if len(single_result) not in (1, 2, 3):
+            raise ValueError(
+                'Expected importer result to be a tuple of length (1, 2, 3) '
+                'but got {}: {!r}'.format(len(single_result), single_result),
+            )
+
+        def _to_bytes(obj):
+            if not isinstance(obj, bytes):
+                return obj.encode('UTF-8')
+            else:
+                return obj
+
+        return tuple(_to_bytes(s) for s in single_result)
+
+    return tuple(_to_importer_result(x) for x in result)
+
+
+def _importer_callback_wrapper(func):
+    def inner(path, prev):
+        path, prev = path.decode('UTF-8'), prev.decode('UTF-8')
+        num_args = getattr(inner, '_num_args', None)
+        if num_args is None:
+            try:
+                ret = func(path, prev)
+            except TypeError:
+                inner._num_args = 1
+                ret = func(path)
+            else:
+                inner._num_args = 2
+        elif num_args == 2:
+            ret = func(path, prev)
+        else:
+            ret = func(path)
+        return _normalize_importer_return_value(ret)
+    return inner
+
+
+def _validate_importers(importers):
+    """Validates the importers and decorates the callables with our output
+    formatter.
+    """
+    # They could have no importers, that's chill
+    if importers is None:
+        return None
+
+    def _to_importer(priority, func):
+        assert isinstance(priority, int), priority
+        assert callable(func), func
+        return (priority, _importer_callback_wrapper(func))
+
+    # Our code assumes tuple of tuples
+    return tuple(_to_importer(priority, func) for priority, func in importers)
+
+
+def _raise(e):
+    raise e
+
+
 def compile_dirname(
     search_path, output_path, output_style, source_comments, include_paths,
-    precision, custom_functions,
+    precision, custom_functions, importers, source_map_contents,
+    source_map_embed, omit_source_map_url, source_map_root,
 ):
     fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
-    for dirpath, _, filenames in os.walk(search_path):
+    for dirpath, _, filenames in os.walk(search_path, onerror=_raise):
         filenames = [
             filename for filename in filenames
-            if filename.endswith(('.scss', '.sass'))
-            and not filename.startswith('_')
+            if filename.endswith(('.scss', '.sass')) and
+            not filename.startswith('_')
         ]
         for filename in filenames:
             input_filename = os.path.join(dirpath, filename)
@@ -161,30 +254,43 @@ def compile_dirname(
             output_filename = os.path.join(output_path, relpath_to_file)
             output_filename = re.sub('.s[ac]ss$', '.css', output_filename)
             input_filename = input_filename.encode(fs_encoding)
-            s, v, _ = compile_filename(
+            s, v, _ = _sass.compile_filename(
                 input_filename, output_style, source_comments, include_paths,
-                precision, None, custom_functions,
+                precision, None, custom_functions, importers, None,
+                source_map_contents, source_map_embed, omit_source_map_url,
+                source_map_root,
             )
             if s:
                 v = v.decode('UTF-8')
                 mkdirp(os.path.dirname(output_filename))
-                with open(output_filename, 'w',
-                          encoding="UTF-8") as output_file:
+                with io.open(
+                    output_filename, 'w', encoding='UTF-8', newline='',
+                ) as output_file:
                     output_file.write(v)
             else:
                 return False, v
     return True, None
 
 
+def _check_no_remaining_kwargs(func, kwargs):
+    if kwargs:
+        raise TypeError(
+            '{}() got unexpected keyword argument(s) {}'.format(
+                func.__name__,
+                ', '.join("'{}'".format(arg) for arg in sorted(kwargs)),
+            ),
+        )
+
+
 def compile(**kwargs):
-    """There are three modes of parameters :func:`compile()` can take:
+    r"""There are three modes of parameters :func:`compile()` can take:
     ``string``, ``filename``, and ``dirname``.
 
-    The ``string`` parameter is the most basic way to compile SASS.
-    It simply takes a string of SASS code, and then returns a compiled
+    The ``string`` parameter is the most basic way to compile Sass.
+    It simply takes a string of Sass code, and then returns a compiled
     CSS string.
 
-    :param string: SASS source code to compile.  it's exclusive to
+    :param string: Sass source code to compile.  it's exclusive to
                    ``filename`` and ``dirname`` parameters
     :type string: :class:`str`
     :param output_style: an optional coding style of the compiled result.
@@ -194,29 +300,42 @@ def compile(**kwargs):
     :param source_comments: whether to add comments about source lines.
                             :const:`False` by default
     :type source_comments: :class:`bool`
+    :param source_map_contents: embed include contents in map
+    :type source_map_contents: :class:`bool`
+    :param source_map_embed: embed sourceMappingUrl as data URI
+    :type source_map_embed: :class:`bool`
+    :param omit_source_map_url: omit source map URL comment from output
+    :type omit_source_map_url: :class:`bool`
+    :param source_map_root: base path, will be emitted in source map as is
+    :type source_map_root: :class:`str`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
-    :param indented: optional declaration that the string is SASS, not SCSS
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: (ignored, for backward compatibility)
+    :param indented: optional declaration that the string is Sass, not SCSS
                      formatted. :const:`False` by default
     :type indented: :class:`bool`
     :returns: the compiled CSS string
+    :param importers: optional callback functions.
+                     see also below `importer callbacks
+                     <importer-callbacks_>`_ description
+    :type importers: :class:`collections.abc.Callable`
     :rtype: :class:`str`
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
 
     The ``filename`` is the most commonly used way.  It takes a string of
-    SASS filename, and then returns a compiled CSS string.
+    Sass filename, and then returns a compiled CSS string.
 
-    :param filename: the filename of SASS source code to compile.
+    :param filename: the filename of Sass source code to compile.
                      it's exclusive to ``string`` and ``dirname`` parameters
     :type filename: :class:`str`
     :param output_style: an optional coding style of the compiled result.
@@ -229,31 +348,42 @@ def compile(**kwargs):
     :param source_map_filename: use source maps and indicate the source map
                                 output filename.  :const:`None` means not
                                 using source maps.  :const:`None` by default.
-                                note that it implies ``source_comments``
-                                is also :const:`True`
     :type source_map_filename: :class:`str`
+    :param source_map_contents: embed include contents in map
+    :type source_map_contents: :class:`bool`
+    :param source_map_embed: embed sourceMappingUrl as data URI
+    :type source_map_embed: :class:`bool`
+    :param omit_source_map_url: omit source map URL comment from output
+    :type omit_source_map_url: :class:`bool`
+    :param source_map_root: base path, will be emitted in source map as is
+    :type source_map_root: :class:`str`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: (ignored, for backward compatibility)
+    :param importers: optional callback functions.
+                     see also below `importer callbacks
+                     <importer-callbacks_>`_ description
+    :type importers: :class:`collections.abc.Callable`
     :returns: the compiled CSS string, or a pair of the compiled CSS string
-              and the source map string if ``source_comments='map'``
+              and the source map string if ``source_map_filename`` is set
     :rtype: :class:`str`, :class:`tuple`
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
     :raises exceptions.IOError: when the ``filename`` doesn't exist or
                                 cannot be read
 
     The ``dirname`` is useful for automation.  It takes a pair of paths.
     The first of the ``dirname`` pair refers the source directory, contains
-    several SASS source files to compiled.  SASS source files can be nested
+    several Sass source files to compiled.  Sass source files can be nested
     in directories.  The second of the pair refers the output directory
     that compiled CSS files would be saved.  Directory tree structure of
     the source directory will be maintained in the output directory as well.
@@ -270,25 +400,34 @@ def compile(**kwargs):
     :param source_comments: whether to add comments about source lines.
                             :const:`False` by default
     :type source_comments: :class:`bool`
+    :param source_map_contents: embed include contents in map
+    :type source_map_contents: :class:`bool`
+    :param source_map_embed: embed sourceMappingUrl as data URI
+    :type source_map_embed: :class:`bool`
+    :param omit_source_map_url: omit source map URL comment from output
+    :type omit_source_map_url: :class:`bool`
+    :param source_map_root: base path, will be emitted in source map as is
+    :type source_map_root: :class:`str`
     :param include_paths: an optional list of paths to find ``@import``\ ed
-                          SASS/CSS source files
-    :type include_paths: :class:`collections.Sequence`, :class:`str`
+                          Sass/CSS source files
+    :type include_paths: :class:`collections.abc.Sequence`
     :param precision: optional precision for numbers. :const:`5` by default.
     :type precision: :class:`int`
     :param custom_functions: optional mapping of custom functions.
                              see also below `custom functions
-                             <custom-functions>`_ description
-    :type custom_functions: :class:`collections.Set`,
-                            :class:`collections.Sequence`,
-                            :class:`collections.Mapping`
+                             <custom-functions_>`_ description
+    :type custom_functions: :class:`set`,
+                            :class:`collections.abc.Sequence`,
+                            :class:`collections.abc.Mapping`
+    :param custom_import_extensions: (ignored, for backward compatibility)
     :raises sass.CompileError: when it fails for any reason
-                               (for example the given SASS has broken syntax)
+                               (for example the given Sass has broken syntax)
 
     .. _custom-functions:
 
     The ``custom_functions`` parameter can take three types of forms:
 
-    :class:`~collections.Set`/:class:`~collections.Sequence` of \
+    :class:`~set`/:class:`~collections.abc.Sequence` of \
     :class:`SassFunction`\ s
        It is the most general form.  Although pretty verbose, it can take
        any kind of callables like type objects, unnamed functions,
@@ -304,7 +443,7 @@ def compile(**kwargs):
               }
           )
 
-    :class:`~collections.Mapping` of names to functions
+    :class:`~collections.abc.Mapping` of names to functions
        Less general, but easier-to-use form.  Although it's not it can take
        any kind of callables, it can take any kind of *functions* defined
        using :keyword:`def`/:keyword:`lambda` syntax.
@@ -321,7 +460,7 @@ def compile(**kwargs):
               }
           )
 
-    :class:`~collections.Set`/:class:`~collections.Sequence` of \
+    :class:`~set`/:class:`~collections.abc.Sequence` of \
     named functions
        Not general, but the easiest-to-use form for *named* functions.
        It can take only named functions, defined using :keyword:`def`.
@@ -337,6 +476,51 @@ def compile(**kwargs):
               custom_functions={func_name}
           )
 
+    .. _importer-callbacks:
+
+    Newer versions of ``libsass`` allow developers to define callbacks to be
+    called and given a chance to process ``@import`` directives. You can
+    define yours by passing in a list of callables via the ``importers``
+    parameter. The callables must be passed as 2-tuples in the form:
+
+    .. code-block:: python
+
+        (priority_int, callback_fn)
+
+    A priority of zero is acceptable; priority determines the order callbacks
+    are attempted.
+
+    These callbacks can accept one or two string arguments. The first argument
+    is the path that was passed to the ``@import`` directive; the second
+    (optional) argument is the previous resolved path, where the ``@import``
+    directive was found. The callbacks must either return ``None`` to
+    indicate the path wasn't handled by that callback (to continue with others
+    or fall back on internal ``libsass`` filesystem behaviour) or a list of
+    one or more tuples, each in one of three forms:
+
+    * A 1-tuple representing an alternate path to handle internally; or,
+    * A 2-tuple representing an alternate path and the content that path
+      represents; or,
+    * A 3-tuple representing the same as the 2-tuple with the addition of a
+      "sourcemap".
+
+    All tuple return values must be strings. As a not overly realistic
+    example:
+
+    .. code-block:: python
+
+        def my_importer(path, prev):
+            return [(path, '#' + path + ' { color: red; }')]
+
+        sass.compile(
+                ...,
+                importers=[(0, my_importer)]
+            )
+
+    Now, within the style source, attempting to ``@import 'button';`` will
+    instead attach ``color: red`` as a property of an element with the
+    imported name.
+
     .. versionadded:: 0.4.0
        Added ``source_comments`` and ``source_map_filename`` parameters.
 
@@ -354,6 +538,17 @@ def compile(**kwargs):
     .. versionadded:: 0.7.0
        Added ``custom_functions`` parameter.
 
+    .. versionadded:: 0.11.0
+       ``source_map_filename`` no longer implies ``source_comments``.
+
+    .. versionadded:: 0.17.0
+       Added ``source_map_contents``, ``source_map_embed``,
+       ``omit_source_map_url``, and ``source_map_root`` parameters.
+
+    .. versionadded:: 0.18.0
+        The importer callbacks can now take a second argument, the previously-
+        resolved path, so that importers can do relative path resolution.
+
     """
     modes = set()
     for mode_name in MODES:
@@ -362,78 +557,100 @@ def compile(**kwargs):
     if not modes:
         raise TypeError('choose one at least in ' + and_join(MODES))
     elif len(modes) > 1:
-        raise TypeError(and_join(modes) + ' are exclusive each other; '
-                        'cannot be used at a time')
+        raise TypeError(
+            and_join(modes) + ' are exclusive each other; '
+            'cannot be used at a time',
+        )
     precision = kwargs.pop('precision', 5)
     output_style = kwargs.pop('output_style', 'nested')
     if not isinstance(output_style, string_types):
-        raise TypeError('output_style must be a string, not ' +
-                        repr(output_style))
+        raise TypeError(
+            'output_style must be a string, not ' +
+            repr(output_style),
+        )
     try:
         output_style = OUTPUT_STYLES[output_style]
     except KeyError:
-        raise CompileError('{0} is unsupported output_style; choose one of {1}'
-                           ''.format(output_style, and_join(OUTPUT_STYLES)))
+        raise CompileError(
+            '{} is unsupported output_style; choose one of {}'
+            ''.format(output_style, and_join(OUTPUT_STYLES)),
+        )
     source_comments = kwargs.pop('source_comments', False)
     if source_comments in SOURCE_COMMENTS:
         if source_comments == 'none':
-            deprecation_message = ('you can simply pass False to '
-                                   "source_comments instead of 'none'")
+            deprecation_message = (
+                'you can simply pass False to '
+                "source_comments instead of 'none'"
+            )
             source_comments = False
         elif source_comments in ('line_numbers', 'default'):
-            deprecation_message = ('you can simply pass True to '
-                                   "source_comments instead of " +
-                                   repr(source_comments))
+            deprecation_message = (
+                'you can simply pass True to '
+                "source_comments instead of " +
+                repr(source_comments)
+            )
             source_comments = True
         else:
-            deprecation_message = ("you don't have to pass 'map' to "
-                                   'source_comments but just need to '
-                                   'specify source_map_filename')
+            deprecation_message = (
+                "you don't have to pass 'map' to "
+                'source_comments but just need to '
+                'specify source_map_filename'
+            )
             source_comments = False
         warnings.warn(
             "values like 'none', 'line_numbers', and 'map' for "
             'the source_comments parameter are deprecated; ' +
             deprecation_message,
-            DeprecationWarning
+            FutureWarning,
         )
     if not isinstance(source_comments, bool):
-        raise TypeError('source_comments must be bool, not ' +
-                        repr(source_comments))
+        raise TypeError(
+            'source_comments must be bool, not ' +
+            repr(source_comments),
+        )
     fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
-    source_map_filename = kwargs.pop('source_map_filename', None)
-    if not (source_map_filename is None or
-            isinstance(source_map_filename, string_types)):
-        raise TypeError('source_map_filename must be a string, not ' +
-                        repr(source_map_filename))
-    elif isinstance(source_map_filename, text_type):
-        source_map_filename = source_map_filename.encode(fs_encoding)
-    if not ('filename' in modes or source_map_filename is None):
-        raise CompileError('source_map_filename is only available with '
-                           'filename= keyword argument since it has to be '
-                           'aware of it')
-    if source_map_filename is not None:
-        source_comments = True
-    try:
-        include_paths = kwargs.pop('include_paths') or b''
-    except KeyError:
-        include_paths = b''
-    else:
-        if isinstance(include_paths, collections.Sequence):
-            include_paths = os.pathsep.join(include_paths)
-        elif not isinstance(include_paths, string_types):
-            raise TypeError('include_paths must be a sequence of strings, or '
-                            'a colon-separated (or semicolon-separated if '
-                            'Windows) string, not ' + repr(include_paths))
-        if isinstance(include_paths, text_type):
-            include_paths = include_paths.encode(fs_encoding)
+
+    def _get_file_arg(key):
+        ret = kwargs.pop(key, None)
+        if ret is not None and not isinstance(ret, string_types):
+            raise TypeError('{} must be a string, not {!r}'.format(key, ret))
+        elif isinstance(ret, text_type):
+            ret = ret.encode(fs_encoding)
+        if ret and 'filename' not in modes:
+            raise CompileError(
+                '{} is only available with filename= keyword argument since '
+                'has to be aware of it'.format(key),
+            )
+        return ret
+
+    source_map_filename = _get_file_arg('source_map_filename')
+    output_filename_hint = _get_file_arg('output_filename_hint')
+
+    source_map_contents = kwargs.pop('source_map_contents', False)
+    source_map_embed = kwargs.pop('source_map_embed', False)
+    omit_source_map_url = kwargs.pop('omit_source_map_url', False)
+    source_map_root = kwargs.pop('source_map_root', None)
+
+    if isinstance(source_map_root, text_type):
+        source_map_root = source_map_root.encode('utf-8')
+
+    # #208: cwd is always included in include paths
+    include_paths = (os.getcwd(),)
+    include_paths += tuple(kwargs.pop('include_paths', ()) or ())
+    include_paths = os.pathsep.join(include_paths)
+    if isinstance(include_paths, text_type):
+        include_paths = include_paths.encode(fs_encoding)
 
     custom_functions = kwargs.pop('custom_functions', ())
-    if isinstance(custom_functions, collections.Mapping):
+    if isinstance(custom_functions, collections_abc.Mapping):
         custom_functions = [
             SassFunction.from_lambda(name, lambda_)
             for name, lambda_ in custom_functions.items()
         ]
-    elif isinstance(custom_functions, (collections.Set, collections.Sequence)):
+    elif isinstance(
+            custom_functions,
+            (collections_abc.Set, collections_abc.Sequence),
+    ):
         custom_functions = [
             func if isinstance(func, SassFunction)
             else SassFunction.from_named_function(func)
@@ -445,20 +662,34 @@ def compile(**kwargs):
             '- a set/sequence of {0.__module__}.{0.__name__} objects,\n'
             '- a mapping of function name strings to lambda functions,\n'
             '- a set/sequence of named functions,\n'
-            'not {1!r}'.format(SassFunction, custom_functions)
+            'not {1!r}'.format(SassFunction, custom_functions),
         )
 
+    if kwargs.pop('custom_import_extensions', None) is not None:
+        warnings.warn(
+            '`custom_import_extensions` has no effect and will be removed in '
+            'a future version.',
+            FutureWarning,
+        )
+
+    importers = _validate_importers(kwargs.pop('importers', None))
+
     if 'string' in modes:
         string = kwargs.pop('string')
         if isinstance(string, text_type):
             string = string.encode('utf-8')
         indented = kwargs.pop('indented', False)
         if not isinstance(indented, bool):
-            raise TypeError('indented must be bool, not ' +
-                            repr(source_comments))
-        s, v = compile_string(
+            raise TypeError(
+                'indented must be bool, not ' +
+                repr(source_comments),
+            )
+        _check_no_remaining_kwargs(compile, kwargs)
+        s, v = _sass.compile_string(
             string, output_style, source_comments, include_paths, precision,
-            custom_functions, indented
+            custom_functions, indented, importers,
+            source_map_contents, source_map_embed, omit_source_map_url,
+            source_map_root,
         )
         if s:
             return v.decode('utf-8')
@@ -467,57 +698,37 @@ def compile(**kwargs):
         if not isinstance(filename, string_types):
             raise TypeError('filename must be a string, not ' + repr(filename))
         elif not os.path.isfile(filename):
-            raise IOError('{0!r} seems not a file'.format(filename))
+            raise IOError('{!r} seems not a file'.format(filename))
         elif isinstance(filename, text_type):
             filename = filename.encode(fs_encoding)
-        s, v, source_map = compile_filename(
+        _check_no_remaining_kwargs(compile, kwargs)
+        s, v, source_map = _sass.compile_filename(
             filename, output_style, source_comments, include_paths, precision,
-            source_map_filename, custom_functions,
+            source_map_filename, custom_functions, importers,
+            output_filename_hint,
+            source_map_contents, source_map_embed, omit_source_map_url,
+            source_map_root,
         )
         if s:
             v = v.decode('utf-8')
             if source_map_filename:
                 source_map = source_map.decode('utf-8')
-                if os.sep != '/' and os.altsep:
-                    # Libsass has a bug that produces invalid JSON string
-                    # literals which contain unescaped backslashes for
-                    # "sources" paths on Windows e.g.:
-                    #
-                    #   {
-                    #     "version": 3,
-                    #     "file": "",
-                    #     "sources": ["c:\temp\tmpj2ac07\test\b.scss"],
-                    #     "names": [],
-                    #     "mappings": "AAAA,EAAE;EAEE,WAAW"
-                    #   }
-                    #
-                    # To workaround this bug without changing libsass'
-                    # internal behavior, we replace these backslashes with
-                    # slashes e.g.:
-                    #
-                    #   {
-                    #     "version": 3,
-                    #     "file": "",
-                    #     "sources": ["c:/temp/tmpj2ac07/test/b.scss"],
-                    #     "names": [],
-                    #     "mappings": "AAAA,EAAE;EAEE,WAAW"
-                    #   }
-                    source_map = re.sub(
-                        r'"sources":\s*\[\s*"[^"]*"(?:\s*,\s*"[^"]*")*\s*\]',
-                        lambda m: m.group(0).replace(os.sep, os.altsep),
-                        source_map
-                    )
                 v = v, source_map
             return v
     elif 'dirname' in modes:
         try:
             search_path, output_path = kwargs.pop('dirname')
         except ValueError:
-            raise ValueError('dirname must be a pair of (source_dir, '
-                             'output_dir)')
+            raise ValueError(
+                'dirname must be a pair of (source_dir, '
+                'output_dir)',
+            )
+        _check_no_remaining_kwargs(compile, kwargs)
         s, v = compile_dirname(
             search_path, output_path, output_style, source_comments,
-            include_paths, precision, custom_functions,
+            include_paths, precision, custom_functions, importers,
+            source_map_contents, source_map_embed, omit_source_map_url,
+            source_map_root,
         )
         if s:
             return
@@ -534,7 +745,7 @@ def and_join(strings):
     'Korea, Japan, China, and Taiwan'
 
     :param strings: a list of words to join
-    :type string: :class:`collections.Sequence`
+    :type string: :class:`collections.abc.Sequence`
     :returns: a joined string
     :rtype: :class:`str`, :class:`basestring`
 
@@ -589,12 +800,17 @@ SASS_SEPARATOR_SPACE = collections.namedtuple('SASS_SEPARATOR_SPACE', ())()
 SEPARATORS = frozenset((SASS_SEPARATOR_COMMA, SASS_SEPARATOR_SPACE))
 
 
-class SassList(collections.namedtuple('SassList', ('items', 'separator'))):
+class SassList(
+    collections.namedtuple(
+            'SassList', ('items', 'separator', 'bracketed'),
+    ),
+):
 
-    def __new__(cls, items, separator):
+    def __new__(cls, items, separator, bracketed=False):
         items = tuple(items)
-        assert separator in SEPARATORS
-        return super(SassList, cls).__new__(cls, items, separator)
+        assert separator in SEPARATORS, separator
+        assert isinstance(bracketed, bool), bracketed
+        return super(SassList, cls).__new__(cls, items, separator, bracketed)
 
 
 class SassError(collections.namedtuple('SassError', ('msg',))):
@@ -613,7 +829,7 @@ class SassWarning(collections.namedtuple('SassWarning', ('msg',))):
         return super(SassWarning, cls).__new__(cls, msg)
 
 
-class SassMap(collections.Mapping):
+class SassMap(collections_abc.Mapping):
     """Because sass maps can have mapping types as keys, we need an immutable
     hashable mapping type.
 
@@ -642,7 +858,7 @@ class SassMap(collections.Mapping):
     # Our interface
 
     def __repr__(self):
-        return '{0}({1})'.format(type(self).__name__, frozenset(self.items()))
+        return '{}({})'.format(type(self).__name__, frozenset(self.items()))
 
     def __hash__(self):
         return self._hash
